# frozen_string_literal: true

module OauthIm
  module Authenticable
    extend ActiveSupport::Concern

    included do
      helper_method :authenticated?
      helper_method :email
    end

    private

    def authenticated?
      AppContext.kh_iiab? && logged_in?
    end

    def email
      @email ||= jwt_token['email']
    end

    def user_jwt
      @user_jwt ||= session[:user_jwt] || {}
    end

    def jwt_token
      @jwt_token ||= user_jwt['value']&.first || {}
    end

    def email_verified?
      jwt_token.present? && jwt_token['email_verified'].present?
    end

    def current_user
      @current_user ||=
        if user_jwt.present?
          if email_verified?
            email
          else
            head :forbidden
          end
        end
    end

    def logged_in?
      current_user.present? || local_login?
    end

    def local_login?
      session[:userinfo].present?
    end
  end
end
