# frozen_string_literal: true

require 'oauth2'

module OauthIm
  class ClientController < OauthIm::ApplicationController
    def callback
      session[:user_jwt] = { value: decoded_token, httponly: true }
      redirect_to main_app.root_path
    rescue StandardError
      head :forbidden
    end

    def login
      redirect_to oauth_client.auth_code.authorize_url
    end

    def logout
      reset_session
      redirect_to logout_uri
    end

    def local_login
      raise 'Disallowed' if Rails.env.production?

      session[:userinfo] = { info: { email: 'local_login@example.com' } }
      redirect_back(fallback_location: main_app.root_path)
    end

    def local_logout
      reset_session
      redirect_back(fallback_location: main_app.root_path)
    end

    private

    def logout_uri
      @logout_uri ||= "#{configuration.domain}/oauth2/logout" \
                      "?post_logout_redirect_uri=#{post_logout_redirect_uri}" \
                      "&client_id=#{configuration.client_id}"
    end

    def post_logout_redirect_uri
      @post_logout_redirect_uri ||= params[:return_to]
    end

    def decoded_token
      @decoded_token ||= TokenDecoder.new(access_token, oauth_client.id).decode
    end

    def access_token
      @access_token ||= response_hash[:access_token]
    end

    def response_hash
      @response_hash ||= auth_code.get_token(params[:code]).to_hash
    end

    def auth_code
      @auth_code ||= oauth_client.auth_code
    end

    def oauth_client
      @oauth_client = ::OAuth2::Client.new configuration.client_id,
                                           configuration.client_secret,
                                           authorize_url: configuration.authorize_url,
                                           site: configuration.domain,
                                           token_url: configuration.token_url,
                                           redirect_uri: callback_url
    end

    def configuration
      OauthIm.configuration
    end
  end
end
