# frozen_string_literal: true

require 'jwt'

module OauthIm
  class TokenDecoder
    attr_reader :token, :aud

    DECODE_ALGORITHM = 'HS256'

    def initialize(token, aud)
      @token = token
      @aud = aud
    end

    def decode
      decoded_token
    end

    private

    def configuration
      @configuration ||= OauthIm.configuration
    end

    def decoded_token
      @decoded_token ||= JWT.decode token, configuration.hmac, true, decode_params
    end

    def decode_params
      @decode_params ||= { verify_iss: true,
                           iss: configuration.iss_domain,
                           verify_aud: true,
                           aud: aud,
                           algorithm: DECODE_ALGORITHM }.freeze
    end
  end
end
